use crate::objects::{AlbumItem, ChunkHeader, DataSet, Database, JumpTable, LetterJumpEntry, Playlist, PlaylistIndexEntry, PlaylistItem, TrackItem};


#[derive(Debug, serde::Serialize)]
pub struct XDatabase {
    pub header: Option<ChunkHeader>,
    pub data: Option<Database>,
    pub children: Vec<XDataSet>
}

#[derive(Debug, serde::Serialize)]
pub struct XDataSet {
    pub header: ChunkHeader,
    pub data: DataSet,
    pub child: XSomeList
}

#[derive(Debug, serde::Serialize, Clone)]
pub struct XTrackItem {
    pub header: ChunkHeader,
    pub data: TrackItem,
    pub args: Vec<XArgument>
}

#[derive(Debug, serde::Serialize)]
pub struct XAlbumItem {
    pub header: ChunkHeader,
    pub data: AlbumItem,
    pub args: Vec<XArgument>
}

#[derive(Debug, serde::Serialize)]
pub struct XPlaylist {
    pub header: ChunkHeader,
    pub data: Playlist,
    pub args: Vec<XPlArgument>,
    pub elems: Vec<(PlaylistItem, Vec<XPlArgument>)>
}

#[derive(Debug, serde::Serialize, Clone)]
pub enum XPlArgument {
    String(XArgument),
    IndexEntry(XPlaylistIndexEntry),
    LetterJumpEntry(XLetterJump),
    RawArgument(Vec<u8>)
}

#[derive(Debug, serde::Serialize, Clone)]
pub struct XPlaylistIndexEntry {
    pub data: PlaylistIndexEntry,
    pub v: Vec<u32>
}

#[derive(Debug, serde::Serialize, Clone)]
pub struct XLetterJump {
    pub data: LetterJumpEntry,
    pub v: Vec<JumpTable>
}

#[derive(Debug, serde::Serialize, Clone)]
pub struct XArgument {
    pub arg_type: u32,
    pub val: String
}

#[derive(Debug, serde::Serialize)]
pub enum XSomeList {
    Playlists(Vec<XPlaylist>),
    TrackList(Vec<XTrackItem>),
    AlbumList(Vec<XAlbumItem>)
}

impl XDatabase {
    pub fn find_dataset(&mut self, data_type: u32) -> &mut XDataSet {
        self.children.iter_mut().find(|d| d.data.data_type == data_type).unwrap()
    }
}